@@ -9,6 +9,11 @@ from orjson import JSONDecodeError, loads
 from pydantic import BaseModel, ValidationError
 
 from repod import errors
+from repod.common.enums import (
+    FilesVersionEnum,
+    OutputPackageVersionEnum,
+    PackageDescVersionEnum,
+)
 from repod.common.models import (
     Arch,
     Backup,
@@ -37,7 +42,28 @@ from repod.common.models import (
     Version,
 )
 from repod.files import package
-from repod.repo.package.syncdb import Files, FilesV1, PackageDesc, PackageDescV1
+from repod.files.buildinfo import (
+    BuildDir,
+    BuildEnv,
+    BuildInfo,
+    BuildInfoV1,
+    BuildInfoV2,
+    BuildTool,
+    BuildToolVer,
+    FormatV1,
+    FormatV2,
+    Installed,
+    Options,
+    PkgBuildSha256Sum,
+    StartDir,
+)
+from repod.repo.package.syncdb import (
+    Files,
+    FilesV1,
+    PackageDesc,
+    PackageDescV1,
+    PackageDescV2,
+)
 
 OUTPUT_PACKAGE_VERSIONS: Dict[int, Dict[str, Set[str]]] = {
     1: {
@@ -77,14 +103,144 @@ OUTPUT_PACKAGE_BASE_VERSIONS: Dict[int, Dict[str, Union[int, Set[str]]]] = {
             "packages",
             "version",
         },
-        "files_version": 1,
-        "output_package_version": 1,
-        "package_desc_version": 1,
     },
 }
 DEFAULT_OUTPUT_PACKAGE_BASE_VERSION = 1
 
 
+class OutputBuildInfo(BaseModel):
+    """A class tracking BuildInfo information of packages that are added to instances of OutputPackageBase
+
+    This class is a base template class and should not be used directly.
+    Instead, instantiate one of its versioned child classes using the `from_buildinfo()` classmethod.
+    """
+
+    @classmethod
+    def from_buildinfo(cls, buildinfo: BuildInfo) -> OutputBuildInfo:
+        """Create and return an instance of one of OutputBuildInfo's versioned child classes
+
+        Parameters
+        ----------
+        buildinfo: BuildInfo
+            An instance of BuildInfo which will be used to create an instance of one of the versioned child classes of
+            OutputBuildInfo
+
+        Raises
+        ------
+        RuntimeError
+            If an unknown BuildInfo instance is encountered
+
+        Returns
+        -------
+        OutputBuildInfo
+            One of OutputBuildInfo's versioned child classes (e.g. OutputBuildInfoV1 or OutputBuildInfoV2)
+        """
+
+        if isinstance(buildinfo, BuildInfoV1):
+            return OutputBuildInfoV1(
+                builddir=buildinfo.builddir,
+                buildenv=buildinfo.buildenv,
+                format_=buildinfo.format_,
+                installed=buildinfo.installed,
+                options=buildinfo.options,
+                pkgbuild_sha256sum=buildinfo.pkgbuild_sha256sum,
+            )
+        elif isinstance(buildinfo, BuildInfoV2):
+            return OutputBuildInfoV2(
+                builddir=buildinfo.builddir,
+                buildenv=buildinfo.buildenv,
+                buildtool=buildinfo.buildtool,
+                buildtoolver=buildinfo.buildtoolver,
+                format_=buildinfo.format_,
+                installed=buildinfo.installed,
+                options=buildinfo.options,
+                pkgbuild_sha256sum=buildinfo.pkgbuild_sha256sum,
+                startdir=buildinfo.startdir,
+            )
+        else:
+            raise RuntimeError(
+                "An unknown input format has been encountered while transforming a package's BuildInfo information "
+                "into the respective output format!"
+            )
+
+
+class OutputBuildInfoV1(
+    BuildDir,
+    BuildEnv,
+    FormatV1,
+    Installed,
+    Options,
+    OutputBuildInfo,
+    PkgBuildSha256Sum,
+):
+    """OutputBuildInfo version 1
+
+    Attributes which are already covered by OutputPackageBase are ommitted.
+    Instances of this class relate to OutputBuildInfo the same way as BuildinfoV1 relates to BuildInfo.
+
+    Attributes
+    ----------
+    builddir: str
+        A string representing an absolute directory
+    buildenv: List[str]
+        A list of strings as described by makepkg.conf's BUILDENV option
+    format_: int
+        An integer describing a BuildInfo format version
+    installed: List[str]
+        A list of strings representing <package_name>-<epoch><version>-<pkgrel>-<architecture> of packages installed
+        during the creation of a package
+    options: List[str]
+        A list of strings representing makepkg.conf OPTIONS used during the creation of a package
+    pkgbuild_sha256sum: str
+        A string representing a SHA-256 checksum for a PKGBUILD of a package
+    """
+
+    pass
+
+
+class OutputBuildInfoV2(
+    BuildDir,
+    BuildEnv,
+    BuildTool,
+    BuildToolVer,
+    FormatV2,
+    Installed,
+    Options,
+    OutputBuildInfo,
+    PkgBuildSha256Sum,
+    StartDir,
+):
+    """OutputBuildInfo version 2
+
+    Attributes which are already covered by OutputPackageBase are ommitted.
+    Instances of this class relate to OutputBuildInfo the same way as BuildinfoV2 relates to BuildInfo.
+
+    Attributes
+    ----------
+    builddir: str
+        A string representing an absolute directory
+    buildenv: List[str]
+        A list of strings as described by makepkg.conf's BUILDENV option
+    buildtool: str
+        The package name of the build tool used to create a package
+    buildtoolver: str
+        The version of the build tool used to create a package
+    format_: int
+        An integer describing a BuildInfo format version
+    installed: List[str]
+        A list of strings representing <package_name>-<epoch><version>-<pkgrel>-<architecture> of packages installed
+        during the creation of a package
+    options: List[str]
+        A list of strings representing makepkg.conf OPTIONS used during the creation of a package
+    pkgbuild_sha256sum: str
+        A string representing a SHA-256 checksum for a PKGBUILD of a package
+    startdir: str
+        A string representing the absolute startdir directory of a package
+    """
+
+    pass
+
+
 class OutputPackage(BaseModel):
     """A template class to describe all required attributes that define a package in the context of an output file
 
@@ -309,17 +465,11 @@ class OutputPackageBase(BaseModel):
                 A subclass of OutputPackage that is the default for the given OutputPackageBase schema_version
             """
 
-            output_versions: List[Optional[int]] = []
-            output_versions.insert(
-                0,
-                OUTPUT_PACKAGE_BASE_VERSIONS[version]["output_package_version"],  # type: ignore[arg-type]
-            )
-            output_versions.insert(
-                1,
-                OUTPUT_PACKAGE_BASE_VERSIONS[version]["files_version"],  # type: ignore[arg-type]
-            )
-            match output_versions:
-                case [1, 1]:
+            outputpackage_version = OutputPackageVersionEnum.DEFAULT.value
+            files_version = FilesVersionEnum.DEFAULT.value
+
+            match (outputpackage_version, files_version):
+                case (1, 1):
                     files = package.get("files")
                     if files:
                         if files.get("schema_version"):
@@ -329,7 +479,10 @@ class OutputPackageBase(BaseModel):
                     return OutputPackageV1(**package)
                 # NOTE: the catch all can never be reached but is here to satisfy our tooling
                 case _:  # pragma: no cover
-                    raise RuntimeError(f"Invalid version ({version} provided for OUTPUT_PACKAGE_BASE_VERSIONS.")
+                    raise RuntimeError(
+                        f"Invalid version provided for OutputPackage ({outputpackage_version} "
+                        f"and/ or Files ({files_version})."
+                    )
 
         used_schema_version = data.get("schema_version")
         if isinstance(used_schema_version, int):
@@ -475,6 +628,7 @@ class OutputPackageBase(BaseModel):
             case 1:
                 return OutputPackageBaseV1(
                     base=packages[0].pkginfo.base,  # type: ignore[attr-defined]
+                    buildinfo=OutputBuildInfo.from_buildinfo(packages[0].buildinfo),  # type: ignore[attr-defined]
                     makedepends=packages[0].pkginfo.makedepends,  # type: ignore[attr-defined]
                     packager=packages[0].pkginfo.packager,  # type: ignore[attr-defined]
                     packages=[OutputPackage.from_package(package=pkg) for pkg in packages],
@@ -534,20 +688,29 @@ class OutputPackageBase(BaseModel):
                 "It is not possible to return the version attribute of the template class OutputPackageBase!"
             )
 
-    async def get_packages_as_models(self) -> List[Tuple[PackageDesc, Files]]:
+    async def get_packages_as_models(
+        self,
+        packagedesc_version: PackageDescVersionEnum = PackageDescVersionEnum.DEFAULT,
+        files_version: FilesVersionEnum = FilesVersionEnum.DEFAULT,
+    ) -> List[Tuple[PackageDesc, Files]]:
         """Return the list of packages as tuples of PackageDesc and Files models
 
-        Depending on the mapping in OUTPUT_PACKAGE_BASE_VERSIONS, which targets the schema_version of OutputPackageBase
-        subclasses, different subclasses of PackageDesc are returned.
-
         NOTE: This method only successfully returns if the instance of the class using it defines the required fields!
         The OutputPackageBase class does not do that!
 
+        Parameters
+        ----------
+        packagedesc_version: int
+            The PackageDesc version to use
+        files_version: int
+            The Files version to use
+
         Raises
         ------
         RuntimeError
-            If an unknown schema_version is encountered in the OutputPackageBase instance
             If this method is called on the template class OutputPackageBase (instead of on one of its subclasses)
+            If an unknown schema_version is encountered in the OutputPackageBase instance.
+            If unsupported packagedesc_version or files_version are encountered.
 
         Returns
         -------
@@ -555,58 +718,89 @@ class OutputPackageBase(BaseModel):
             A list of tuples with one PackageDesc and one Files each
         """
 
-        if hasattr(self, "schema_version"):
-            output_versions: List[Optional[int]] = []
-            schema_version = OUTPUT_PACKAGE_BASE_VERSIONS.get(self.schema_version)  # type: ignore[attr-defined]
-            if schema_version:
-                output_versions.insert(0, schema_version.get("package_desc_version"))  # type: ignore[arg-type]
-                output_versions.insert(1, schema_version.get("files_version"))  # type: ignore[arg-type]
-
-            match output_versions:
-                case [1, 1]:
-                    return [
-                        (
-                            PackageDescV1(
-                                arch=package.arch,
-                                backup=package.backup,
-                                base=self.base,  # type: ignore[attr-defined]
-                                builddate=package.builddate,
-                                checkdepends=package.checkdepends,
-                                conflicts=package.conflicts,
-                                csize=package.csize,
-                                depends=package.depends,
-                                desc=package.desc,
-                                filename=package.filename,
-                                groups=package.groups,
-                                isize=package.isize,
-                                license=package.license,
-                                makedepends=self.makedepends,  # type: ignore[attr-defined]
-                                md5sum=package.md5sum,
-                                name=package.name,
-                                optdepends=package.optdepends,
-                                packager=self.packager,  # type: ignore[attr-defined]
-                                pgpsig=package.pgpsig,
-                                provides=package.provides,
-                                replaces=package.replaces,
-                                sha256sum=package.sha256sum,
-                                url=package.url,
-                                version=self.version,  # type: ignore[attr-defined]
-                            ),
-                            FilesV1(files=package.files.files if package.files else []),
-                        )
-                        for package in self.packages  # type: ignore[attr-defined]
-                    ]
-                case _:
-                    raise RuntimeError(
-                        f"Unknown schema_version ({self.schema_version}) provided "  # type: ignore[attr-defined]
-                        "while attempting to retrieve packages and their files from an instance of "
-                        f"'{self.__class__.__name__}'!"
-                    )
-        else:
+        if not hasattr(self, "schema_version"):
             raise RuntimeError(
                 "Packages and their files can not be retrieved from the templatae class OutputPackageBase!"
             )
 
+        if self.schema_version not in OUTPUT_PACKAGE_BASE_VERSIONS.keys():  # type: ignore[attr-defined]
+            raise RuntimeError(
+                f"OutputPackageBase has invalid schema_version {self.schema_version}!"  # type: ignore[attr-defined]
+            )
+
+        match (packagedesc_version, files_version):
+            case (PackageDescVersionEnum.ONE, FilesVersionEnum.DEFAULT):
+                return [
+                    (
+                        PackageDescV1(
+                            arch=package.arch,
+                            backup=package.backup,
+                            base=self.base,  # type: ignore[attr-defined]
+                            builddate=package.builddate,
+                            checkdepends=package.checkdepends,
+                            conflicts=package.conflicts,
+                            csize=package.csize,
+                            depends=package.depends,
+                            desc=package.desc,
+                            filename=package.filename,
+                            groups=package.groups,
+                            isize=package.isize,
+                            license=package.license,
+                            makedepends=self.makedepends,  # type: ignore[attr-defined]
+                            md5sum=package.md5sum,
+                            name=package.name,
+                            optdepends=package.optdepends,
+                            packager=self.packager,  # type: ignore[attr-defined]
+                            pgpsig=package.pgpsig,
+                            provides=package.provides,
+                            replaces=package.replaces,
+                            sha256sum=package.sha256sum,
+                            url=package.url,
+                            version=self.version,  # type: ignore[attr-defined]
+                        ),
+                        FilesV1(files=package.files.files if package.files else []),
+                    )
+                    for package in self.packages  # type: ignore[attr-defined]
+                ]
+            case (PackageDescVersionEnum.TWO, FilesVersionEnum.DEFAULT):
+                return [
+                    (
+                        PackageDescV2(
+                            arch=package.arch,
+                            backup=package.backup,
+                            base=self.base,  # type: ignore[attr-defined]
+                            builddate=package.builddate,
+                            checkdepends=package.checkdepends,
+                            conflicts=package.conflicts,
+                            csize=package.csize,
+                            depends=package.depends,
+                            desc=package.desc,
+                            filename=package.filename,
+                            groups=package.groups,
+                            isize=package.isize,
+                            license=package.license,
+                            makedepends=self.makedepends,  # type: ignore[attr-defined]
+                            md5sum=package.md5sum,
+                            name=package.name,
+                            optdepends=package.optdepends,
+                            packager=self.packager,  # type: ignore[attr-defined]
+                            provides=package.provides,
+                            replaces=package.replaces,
+                            sha256sum=package.sha256sum,
+                            url=package.url,
+                            version=self.version,  # type: ignore[attr-defined]
+                        ),
+                        FilesV1(files=package.files.files if package.files else []),
+                    )
+                    for package in self.packages  # type: ignore[attr-defined]
+                ]
+            # NOTE: we can never reach this unless we patch PackageDescVersionEnum and/ or FilesVersionEnum
+            case _:  # pragma: no cover
+                raise RuntimeError(
+                    f"Invalid versions provided for Files ({files_version.value}) and/ or "
+                    f"PackageDesc ({packagedesc_version.value}) provided!"
+                )
+
 
 class OutputPackageBaseV1(
     OutputPackageBase,
@@ -624,6 +818,9 @@ class OutputPackageBaseV1(
     base: str
         The attribute can be used to describe the (required) data below a %BASE% identifier in a 'desc' file, which
         identifies a package's pkgbase
+    buildinfo: Optional[OutputBuildInfo]
+        An optional OutputBuildInfo, which describes the build circumstances of the OutputPackageBase. The data is not
+        covered in a repository sync database and therefore optional.
     makedepends: Optional[List[str]]
         The attribute can be used to describe the (optional) data below a %MAKEDEPENDS% identifier in a 'desc' file,
         which identifies a package's makedepends
@@ -639,6 +836,7 @@ class OutputPackageBaseV1(
         identifies a package's version (this is the accumulation of epoch, pkgver and pkgrel)
     """
 
+    buildinfo: Optional[OutputBuildInfo]
     packages: List[OutputPackage]
 
 
@@ -656,7 +854,7 @@ def export_schemas(output: Union[Path, str]) -> None:
         If output is not an existing directory
     """
 
-    classes = [OutputPackageV1, OutputPackageBaseV1]
+    classes = [OutputBuildInfoV1, OutputBuildInfoV2, OutputPackageV1, OutputPackageBaseV1]
 
     if isinstance(output, str):
         output = Path(output)
